/*
 * duck.hpp
 *
 *  Created on: 18 окт. 2021 г.
 *      Author: alexander
 */

#pragma once

#include <iostream>

class Duck
{
protected:
    FlyBehavior *flyBehavior;
    QuackBehavior *quackBehavior;
public:
    void performQuack()
    {
        quackBehavior->quack();
    }

    void performFly()
    {
        flyBehavior->fly();
    }

    void swim()
    {
        std::cout << "All ducks float, even decoys!" << std::endl;
    }

    virtual void display() { }

    void setFlyBehavior(FlyBehavior *fb)
    {
        if (flyBehavior)
        {
            delete flyBehavior;
        }

        flyBehavior = fb;
    }

    void setQuackBehavior(QuackBehavior *qb)
    {
        if (quackBehavior)
        {
            delete quackBehavior;
        }

        quackBehavior = qb;
    }

    virtual ~Duck()
    {
        if (flyBehavior)
        {
            delete flyBehavior;
        }

        if (quackBehavior)
        {
            delete quackBehavior;
        }
    }
};

class MallardDuck : public Duck
{
public:
    MallardDuck()
    {
        quackBehavior = new Quack;
        flyBehavior = new FlyWithWings;
    }
};

class ModelDuck : public Duck
{
public:
    ModelDuck()
    {
        flyBehavior = new FlyNoWay;
        quackBehavior = new Quack;
    }

    void display()
    {
        std::cout << "I'm a model duck" << std::endl;
    }
};
